pub enum StoreError {
InvalidKey(String),
ValueTooLarge,
StoreFull,
SerializationError(String),
InvalidSnapshot,
IoError(String),
NetworkError(String),
ConsensusError(String),
Timeout,
ShuttingDown,
Internal(String),
}Expand description
Errors that can occur during KVStore operations
Variants
InvalidKey(String)
Invalid key provided
ValueTooLarge
Value is too large to store
StoreFull
Store has reached maximum capacity
SerializationError(String)
Serialization error
InvalidSnapshot
Invalid snapshot
IoError(String)
IO error
NetworkError(String)
Network error
ConsensusError(String)
Consensus error
Timeout
Timeout error
ShuttingDown
Store is shutting down
Internal(String)
Internal error
Implementations
Sourceimpl StoreError
impl StoreError
Sourcepub fn is_recoverable(&self) -> bool
pub fn is_recoverable(&self) -> bool
Check if this error is recoverable
Sourcepub fn is_client_error(&self) -> bool
pub fn is_client_error(&self) -> bool
Check if this error indicates a client error (4xx equivalent)
Sourcepub fn is_server_error(&self) -> bool
pub fn is_server_error(&self) -> bool
Check if this error indicates a server error (5xx equivalent)